@@ -39,11 +39,11 @@ yahoo_account_use_http_proxy(PurpleConnection *pc)
 	PurpleProxyInfo *ppi = NULL;
 	PurpleProxyType type = PURPLE_PROXY_NONE;
 	gboolean proxy_ssl = purple_account_get_bool(account, "proxy_ssl", FALSE);
-	
+
 	if(proxy_ssl)
 		ppi = purple_proxy_get_setup(account);
 	else
-		ppi = purple_global_proxy_get_info();
+		ppi = purple_proxy_get_setup(NULL);
 
 	type = purple_proxy_info_get_type(ppi);
 
